@@ -2,7 +2,7 @@
 
 #  Simon Drabble	03/22/02
 #  sdrabble@cpan.org
-#  $Id: snagmail.pl,v 1.3 2002/08/10 03:07:16 simon Exp $
+#  $Id: snagmail.pl,v 1.6 2002/10/24 21:39:23 simon Exp $
 #
 
 
@@ -13,12 +13,14 @@ use Mail::Webmail::Yahoo;
 my $name = $ARGV[0] or &usage, die;
 my $pass = $ARGV[1] or &usage, die;
 
-my $prog = $0;
 
-## Hide the password.
+## Hide the password.. doesn't work on some systems. Also may not be able to
+## assign to $0, so copy it first.
+my $prog = $0;
 $prog =~ s/\s.*//g;
 $0 = $prog;
 
+
 my $yahoo = new Mail::Webmail::Yahoo(
 		username => $name,
 		password => $pass,
@@ -28,15 +30,21 @@ my $yahoo = new Mail::Webmail::Yahoo(
 $| = 1;
 
 
-$yahoo->trace(6);
+# Set this to a positive number for more debugging.
+$yahoo->trace(1);
 
 my @fetchnums;
-if ($ARGV[2]) {
+my $msg_list;
+if (lc $ARGV[2] eq 'all') {
+	$msg_list = 'all';
+} elsif ($ARGV[2]) {
 	eval { @fetchnums = (eval $ARGV[2]) };
+	$msg_list = \@fetchnums;
 }
 
-my @messages = $yahoo->get_mail_messages('Inbox', \@fetchnums);
+my @messages = $yahoo->get_mail_messages('Inbox', $msg_list);
 print "Message Headers in Inbox: ", 0+@messages, "\n";
+print "Messages will be delivered to ./${name}_Inbox.\n";
 open INBOX, ">${name}_Inbox";
 for (@messages) { print INBOX $_->as_mbox_string }
 close INBOX;
@@ -51,3 +59,6 @@ sub usage
 $0 <yahoo username> <yahoo password>  [<number of messages to download>]
 };	
 }
+
+
+
